@@ -3,7 +3,8 @@ package Apache2::AuthCookie;
 use strict;
 
 use Carp;
-use mod_perl2 qw(1.9922 StackedHandlers MethodHandlers Authen Authz);
+use CGI '3.12';
+use mod_perl2 '1.9922';
 
 use Apache::AuthCookie::Util;
 use Apache2::RequestRec;
@@ -12,13 +13,12 @@ use Apache2::Log;
 use Apache2::Access;
 use Apache2::Response;
 use Apache2::Util;
-use Apache2::URI;
 use APR::Table;
 use Apache2::Const qw(:common M_GET HTTP_FORBIDDEN HTTP_MOVED_TEMPORARILY);
 use vars qw($VERSION);
 
-# $Id: AuthCookie.pm,v 1.7 2005/04/19 02:36:45 mschout Exp $
-$VERSION = '3.08';
+# $Id: AuthCookie.pm,v 1.18 2006/05/04 05:08:25 mschout Exp $
+$VERSION = '3.09';
 
 sub recognize_user {
     my ($self, $r) = @_;
@@ -73,7 +73,6 @@ sub cookie_name {
 
 sub handle_cache {
     my ($self, $r) = @_;
-    _check_request_rec(@_);
 
     my $auth_name = $r->auth_name;
 
@@ -87,7 +86,6 @@ sub handle_cache {
 
 sub remove_cookie {
     my ($self, $r) = @_;
-    _check_request_rec(@_);
 
     my $cookie_name = $self->cookie_name($r);
 
@@ -123,7 +121,9 @@ sub _convert_to_get {
              or $name =~ /^credential_\d+$/;
 
         $value = '' unless defined $value;
-        push @pairs, escape_uri($r, $name) . '=' . escape_uri($r, $value);
+        for my $v (split /\0/, $value) {
+            push @pairs, escape_uri($r, $name) . '=' . escape_uri($r, $v);
+        }
     }
 
     $r->args(join '&', @pairs) if scalar(@pairs) > 0;
@@ -144,33 +144,9 @@ sub _get_form_data {
 
     my $data = '';
 
-    if ($r->method eq 'POST') {
-        # $r->content doesnt exist anymore. This is the Apache::compat
-        # version.  Maybe there is more efficient way to do this...
-        $r->setup_client_block;
+    my $cgi = CGI->new($r);
 
-        return unless $r->should_client_block;
-
-        my $buf;
-        while (my $read_len = $r->get_client_block($buf, 8192)) {
-            if ($read_len == -1) {
-                die "get_client_block() error";
-            }
-            $data .= $buf;
-        }
-    }
-    else {
-        $data = $r->args;
-    }
-
-    my %args = ();
-
-    if (defined $data) {
-        %args = map { Apache2::URI::unescape_url($_) }
-                split /[=&;]/, $data;
-    }
-
-    return %args;
+    return $cgi->Vars();
 }
 
 sub login {
@@ -179,6 +155,7 @@ sub login {
     my $debug = $r->dir_config("AuthCookieDebug") || 0;
 
     my $auth_type = $r->auth_type;
+    my $auth_name = $r->auth_name;
 
     my %args = $self->_get_form_data($r);
 
@@ -194,12 +171,15 @@ sub login {
 
     # Get the credentials from the data posted by the client
     my @credentials;
-    while (exists $args{"credential_" . scalar(@credentials)}) {
-        my $key = 'credential_'.scalar(@credentials);
+    for (my $i = 0; exists $args{"credential_$i"}; $i++) {
+        my $key = "credential_$i";
         $r->server->log_error("$key $args{$key}") if $debug >= 2;
         push @credentials, $args{$key};
     }
 
+    # save creds in pnotes so login form script can use them if it wants to
+    $r->pnotes("${auth_name}Creds", \@credentials);
+
     # Exchange the credentials for a session key.
     my $ses_key = $self->authen_cred($r, @credentials);
     unless ($ses_key) {
@@ -245,7 +225,8 @@ sub authenticate {
     $r->server->log_error("auth_type " . $auth_type) if ($debug >= 3);
 
     unless ($r->is_initial_req) {
-        # only authenticate the first internal redirect
+        # we are in a subrequest.  Just copy user from previous request.
+        $r->user( $r->prev->user );
         return OK;
     }
 
@@ -319,7 +300,6 @@ sub authenticate {
 
 sub login_form {
     my ($self, $r) = @_;
-    _check_request_rec(@_);
 
     my $auth_name = $r->auth_name;
 
@@ -439,7 +419,6 @@ sub authorize {
 
 sub send_cookie {
     my ($self, $r, $ses_key, $cookie_args) = @_;
-    _check_request_rec(@_);
 
     $cookie_args = {} unless defined $cookie_args;
 
@@ -511,7 +490,6 @@ sub cookie_string {
 
 sub key {
     my ($self, $r) = @_;
-    _check_request_rec(@_);
 
     my $cookie_name = $self->cookie_name($r);
 
@@ -528,24 +506,6 @@ sub get_cookie_path {
     return $r->dir_config("${auth_name}Path");
 }
 
-# this just checks that the second arg is an Apache::RequestRec object.
-# if not, an exception is thrown.
-sub _check_request_rec {
-    my ($self, $r) = @_;
-
-    my $func = (caller)[3];
-
-    unless (defined $r) {
-        croak "missing Apache2::RequestRec argument to $func.",
-              "module not ported to mod_perl v2?";
-    }
-
-    unless (ref $r and $r->isa('Apache2::RequestRec')) {
-        croak "'$r' is not an Apache2::RequestRec object.",
-              "module not ported to mod_perl v2?";
-    }
-}
-
 1;
 
 __END__
@@ -621,7 +581,7 @@ MethodHandlers, Authen, and Authz compiled in.
   AuthType Sample::AuthCookieHandler
   AuthName WhatEver
   SetHandler perl-script
-  PerlHandler Sample::AuthCookieHandler->login
+  PerlResponseHandler Sample::AuthCookieHandler->login
  </Files>
 
 =head1 DESCRIPTION
@@ -711,6 +671,18 @@ command
 into your server setup file and your cookies for this AuthCookie realm will be
 named MyCustomName.  Default is AuthType_AuthName.
 
+=item 6.
+
+By default users must satisfy ALL of the C<require> directives.  If you
+want authentication to succeed if ANY C<require> directives are met, use the
+C<Satisfy> directive.  For instance, if your AuthName is C<WhatEver>, you can
+put the command
+
+ PerlSetVar WhatEverSatisfy Any
+
+into your server startup file and authentication for this realm will succeed if
+ANY of the C<require> directives are met.
+
 =back
 
 This is the flow of the authentication handler, less the details of the
@@ -817,12 +789,6 @@ called as C<$r-E<gt>species($r, $args)>, where C<$args> is everything
 on your C<require> line after the word C<species>.  The method should
 return OK on success and HTTP_FORBIDDEN on failure.
 
-Currently users must satisfy ALL of the C<require> directives.  I have
-heard that other Apache modules let the user satisfy ANY of the
-C<require> directives.  I don't know which is correct, I haven't found
-any Apache docs on the matter.  If you need one behavior or the other,
-be careful.  I may change it if I discover that ANY is correct.
-
 =item * authen_cred()
 
 You must define this method yourself in your subclass of
@@ -1071,7 +1037,7 @@ implement anything, though.
 
 =head1 CVS REVISION
 
-$Id: AuthCookie.pm,v 1.7 2005/04/19 02:36:45 mschout Exp $
+$Id: AuthCookie.pm,v 1.18 2006/05/04 05:08:25 mschout Exp $
 
 =head1 AUTHOR
 
